
/* GKrellM Powerbook PMU plugin 
|  Battery monitor using PMUD for Apple Powerbooks running Linux/PMac
|  Copyright (C) 2000  Joseph Palani Garcia
|  Author:   Joseph Palani Garcia jpgarcia@execpc.com
|  Version:  0.9
|  Latest versions might be found at:  http://www.execpc.com/~jpgarcia
|  or by following links from the GKrellM homepage at: http://gkrellm.net
|
|  This program is free software which I release under the GNU General Public
|  License. You may redistribute and/or modify this program under the terms
|  of that license as published by the Free Software Foundation; either
|  version 2 of the License, or (at your option) any later version.
|
|  This program is distributed in the hope that it will be useful,
|  but WITHOUT ANY WARRANTY; without even the implied warranty of
|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|  GNU General Public License for more details.
|
|  To get a copy of the GNU General Public License,  write to the
|  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "powerbook_pmu_priv.h"
#include "powerbook_pmu.h"

_sys_pmu sys_pmu;

gint get_current ()
{
     return (gint) (sys_pmu.b[1].current + sys_pmu.b[0].current);
}

void
read_keylargo_pmu ()
{
}


void
read_paddington_pmu ()
{	/* heathrow and paddington are the same */
     signed short batt[5];
     unsigned char par;

     char *b;
     char c[150];

     sys_pmu.time_left = 0;
     c[0] = '\n';
     write (sys_pmu.pmud.sock, c, 1);
     read (sys_pmu.pmud.sock, c, 150);
     b = strtok (c, PMUD_INPUT_DELIM);
     if (b[0] != 'S')
	  return;
     for (par = 1; par <= sys_pmu.nbattery; par++) {
	  b = strtok (0, PMUD_INPUT_DELIM);
	  if (b[4] == '}') {
	       b[4] = b[5];
	       batt[0] = atoi (++b);
	  } else {
	       batt[0] = atoi (++b);
	       batt[1] = atoi (strtok (0, PMUD_INPUT_DELIM));
	       batt[2] = atoi (strtok (0, PMUD_INPUT_DELIM));
	       batt[3] = atoi (strtok (0, PMUD_INPUT_DELIM));
	       b = strtok (0, PMUD_INPUT_DELIM);
	       b[strlen (b) - 1] = '\0';
	       batt[4] = atoi (b);
	  }
	  if ((batt[0] % 10) == 1) {
	       sys_pmu.b[par - 1].available = BATT_YES;
	       sys_pmu.b[par - 1].percentage = batt[1] * 100 / batt[2];
	       sys_pmu.b[par - 1].current = batt[3];
	       sys_pmu.b[par - 1].charging = (((batt[0] / 10) % 10) == 1);
	       sys_pmu.time_left += (((batt[0] / 100) == 1) ? batt[2] - batt[1] : batt[1]);
	  } else {
	       sys_pmu.b[par - 1].available = BATT_NO;
	       sys_pmu.b[par - 1].percentage = 0;
	       sys_pmu.b[par - 1].current = 0;
	       sys_pmu.b[par - 1].charging = 0;
	  }
     }
     if (get_current () < 0)
	  sys_pmu.time_left = sys_pmu.time_left * 3552 / (-(get_current ()) * 60);
     else if (sys_pmu.show_charge_time && (get_current () > 0))
	  sys_pmu.time_left = sys_pmu.time_left * 3552 / (get_current () * 60);
     else
	  sys_pmu.time_left = -1;
     sys_pmu.ac_is_on_line = ((batt[0] / 100) == 1);

}

void
read_ohare_pmu ()
{
#if 0
     unsigned char reply[8];
     float charge, pcharge;
     float vb, i;
     int j;
     int lrange[] = { 0, 275, 850, 1680, 2325, 2765, 3160, 3500, 3830, 4115, 4360,
	  4585, 4795, 4990, 5170, 5340, 5510, 5710, 5930, 6150, 6370,
	  6500
     };

     sys_pmu.ac_is_on_line = reply[0] & 0x01;
     if (reply[0] & 0x04) {
	  sys_pmu.b[0].available = BATT_YES;
	  sys_pmu.b[0].charging = reply[0] & 0x02;
	  sys_pmu.b[0].current = reply[5];
	  vb = (float) ((reply[1] << 8) + reply[2]);	/* battery voltage */
	  if (!sys_pmu.ac_is_on_line) {
	       if (sys_pmu.b[0].current > 200)
		    vb = vb + (float) (sys_pmu.b[0].current - 200) * 0.15;
	  } else if (sys_pmu.b[0].charging)
	       vb = vb - 10.0;
	  i = (330.0 - vb) / 10.0;
	  j = (int) (i);
	  if (j <= 0)
	       charge = 0.0;
	  else if (j >= 21)
	       charge = 6500.0;
	  else
	       charge = (float) (lrange[j] + (i - j) * (lrange[j + 1] - lrange[j]));
	  charge = (float) (1000.0 - charge / 6.5) / 10.0;
	  if (!sys_pmu.ac_is_on_line) {
	       pcharge = (reply[6] << 8) + reply[7];
	       if (pcharge > 6500.0)
		    pcharge = 6500.0;
	       pcharge = (1000.0 - pcharge / 6.5) / 10.0;
	       if (pcharge < charge)
		    charge = pcharge;
	  }
	  if (!sys_pmu.ac_is_on_line) {
	       if (sys_pmu.b[0].current > 0)
		    sys_pmu.time_left = ((int) (charge * 274.0)) / sys_pmu.b[0].current;
	       else
		    sys_pmu.time_left = -1;
	  }
	  sys_pmu.b[0].percentage = (int) charge;
	  if (sys_pmu.b[0].percentage > 100)
	       sys_pmu.b[0].percentage = 100;
     } else {
	  sys_pmu.b[0].available = BATT_NO;
	  sys_pmu.b[0].percentage = 0;
	  sys_pmu.b[0].current = 0;
	  sys_pmu.b[0].charging = 0;
     }
#endif
}
