
/* GKrellM Powerbook PMU plugin 
|  Battery monitor using PMUD for Apple Powerbooks running Linux/PMac
|  Copyright (C) 2000  Joseph Palani Garcia
|  Author:   Joseph Palani Garcia jpgarcia@execpc.com
|  Version:  0.9
|  Latest versions might be found at:  http://www.execpc.com/~jpgarcia
|  or by following links from the GKrellM homepage at: http://gkrellm.net
|
|  This program is free software which I release under the GNU General Public
|  License. You may redistribute and/or modify this program under the terms
|  of that license as published by the Free Software Foundation; either
|  version 2 of the License, or (at your option) any later version.
|
|  This program is distributed in the hope that it will be useful,
|  but WITHOUT ANY WARRANTY; without even the implied warranty of
|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|  GNU General Public License for more details.
|
|  To get a copy of the GNU General Public License,  write to the
|  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "powerbook_pmu_priv.h"
#include "powerbook_pmu.h"

#if USE_CUSTOM_DECALS
#include CUSTOM_DECALS_INC
#else
#include "powerbook_pmu.xpm"
#endif

struct pmuThemeConfig {
     GdkImlibImage *decal_pmu_image;
     GdkPixmap *decal_pmu_pixmap;
     GdkBitmap *decal_pmu_mask;
} PK;
Panel *pmu;

gint pmu_button_release (GtkWidget * widget, GdkEventExpose * event)
{
     sys_pmu.draw_current = (sys_pmu.draw_current ? 0 : 1);
     return FALSE;
}
void
pmu_draw_text_decal (int do_current, Decal * d)
{
     gchar buf[7];
     gint x, w;
     gint t;

     if (do_current)
	  t = get_current ();
     else
	  t = sys_pmu.time_left;

     if (t == d->value)
	  return;

     /* Start showing minutes left for last 500 minutes
      */
     if (do_current)
	  sprintf (buf, "%4d mA", t);
     else if (t == -1)
	  sprintf (buf, "-:--");
     else
	  sprintf (buf, "%d:%02d", t / 60, t % 60);

     w = gdk_string_measure (d->text_style.font, buf);
     x = (d->w - w) / 2;
     if (x < 0)
	  x = 0;
     gdk_draw_pixmap (d->pixmap, gkrellm_draw_GC (1), pmu->bg_pixmap, d->x, d->y, 0, 0, d->w,
	  d->h);
     gkrellm_draw_string (d->pixmap, &d->text_style, x, d->y_baseline, buf);
     d->modified = TRUE;
     d->value = t;
}

void
set_decal (Decal * d, gint val)
{
     if (val == d->value)
	  return;
     gkrellm_draw_decal_pixmap (pmu, d, val);
     d->modified = TRUE;
     d->value = val;
}

void
update_battery_decals ()
{
     int c;

     for (c = 0; c < (sys_pmu.left_mode_on ? 1 : sys_pmu.nbattery); c++) {
	  if (sys_pmu.b[c].available == BATT_NO) {
	       if (sys_pmu.ac_is_on_line)
		    set_decal (sys_pmu.b[c].icon, BATT_NA);
	       else
		    set_decal (sys_pmu.b[c].icon, BATT_NO);
	  } else if (sys_pmu.b[c].percentage == 100)
	       set_decal (sys_pmu.b[c].icon, BATT_FULL);
	  else if (sys_pmu.b[c].percentage > 20)
	       set_decal (sys_pmu.b[c].icon, BATT_OK);
	  else
	       set_decal (sys_pmu.b[c].icon, BATT_LOW);

	  if (sys_pmu.b[c].charging)
	       set_decal (sys_pmu.b[c].modif, BATT_CHARGE);
	  else if (sys_pmu.ac_is_on_line)
	       set_decal (sys_pmu.b[c].modif, BATT_AC);
	  else if (sys_pmu.b[c].current < -300)
	       set_decal (sys_pmu.b[c].modif, BATT_USE);
	  else
	       set_decal (sys_pmu.b[c].modif, BATT_NA);
     }
}

void
update_krells ()
{
     int c;

     for (c = 0; c < (sys_pmu.left_mode_on ? 1 : sys_pmu.nbattery); c++) {
	  sys_pmu.b[c].krell->previous = 0;
	  gkrellm_update_krell (pmu, sys_pmu.b[c].krell, sys_pmu.b[c].percentage);
     }
}

void
update_text_decals ()
{
     switch (sys_pmu.nbattery) {
	  case 2:
	  if (!sys_pmu.left_mode_on) {
	       pmu_draw_text_decal (0, sys_pmu.b[0].text);
	       pmu_draw_text_decal (1, sys_pmu.b[1].text);
	  } else
	  case 1:
	       pmu_draw_text_decal (sys_pmu.draw_current, sys_pmu.b[0].text);
	  break;
	  default:
	  break;
     }
}



void
create_pmu_krells ()
{
     gint y;
     int c;
     gint meter_style;
     Style *style;

     style = gkrellm_meter_style (APM_STYLE);
     y = style->margin;
     gkrellm_destroy_krell_list (pmu);
     for (c = 0; c < (sys_pmu.left_mode_on ? 1 : sys_pmu.nbattery); c++) {
	  if (sys_pmu.use_memory_style)
	       meter_style = ((c == 1) ? SWAP_STYLE : MEM_STYLE);
	  else
	       meter_style = FS_STYLE;

	  if (c == 1) {
	       Style kstyl;

	       memcpy (&kstyl, (gkrellm_meter_style (meter_style)), sizeof (kstyl));
	       if (sys_pmu.low_second_krell == TRUE)
		    kstyl.krell_yoff =
			 kstyl.krell_yoff + y + sys_pmu.b[c].icon->h -
			 sys_pmu.b[0].krell->h_frame;
	       else
		    kstyl.krell_yoff = kstyl.krell_yoff + y;
	       sys_pmu.b[c].krell =
		    gkrellm_create_krell (pmu, gkrellm_krell_meter_image (meter_style),
		    &kstyl);
	  } else {
	       sys_pmu.b[c].krell =
		    gkrellm_create_krell (pmu, gkrellm_krell_meter_image (meter_style),
		    gkrellm_meter_style (meter_style));
	  }
	  sys_pmu.b[c].krell->full_scale = 100;
	  y = sys_pmu.b[c].icon->y + sys_pmu.b[c].icon->h + VERT_MARGIN;
     }
}

gint pmu_expose_event (GtkWidget * widget, GdkEventExpose * event)
{
     if (widget == pmu->drawing_area) {
	  gdk_draw_pixmap (widget->window,
	       widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
	       pmu->pixmap,
	       event->area.x, event->area.y,
	       event->area.x, event->area.y, event->area.width, event->area.height);
     }
     return FALSE;
}

void
create_pmu (GtkWidget * vbox, gint first_create)
{
     Style *style;
     gint x, w, y;
     int c;

     if (first_create) {
#if USE_CUSTOM_DECALS
	  PK.decal_pmu_image = gdk_imlib_create_image_from_xpm_data (CUSTOM_DECALS_DATA);
#else
	  PK.decal_pmu_image = gdk_imlib_create_image_from_xpm_data (powerbook_pmu_xpm);
#endif
	  render_to_pixmap (PK.decal_pmu_image, &PK.decal_pmu_pixmap, &PK.decal_pmu_mask, 0,
	       0);
	  pmu = gkrellm_panel_new0 ();
     } else {
	  gkrellm_destroy_decal_list (pmu);
	  gkrellm_destroy_krell_list (pmu);
     }

     style = gkrellm_meter_style (APM_STYLE);
     pmu->textstyle = gkrellm_meter_textstyle (APM_STYLE);

     y = style->margin;

     for (c = 0; c < (sys_pmu.left_mode_on ? 1 : sys_pmu.nbattery); c++) {
	  sys_pmu.b[c].icon = gkrellm_create_decal_pixmap (pmu, PK.decal_pmu_pixmap,
	       PK.decal_pmu_mask, BATT_NDECALS, style, -1, -1);
	  sys_pmu.b[c].modif = gkrellm_create_decal_pixmap (pmu, PK.decal_pmu_pixmap,
	       PK.decal_pmu_mask, BATT_NDECALS, style, -1, -1);

	  w = gkrellm_chart_width () - (style->margin + sys_pmu.b[c].icon->w + HORIZ_MARGIN);
	  x = sys_pmu.b[c].icon->w + style->margin + HORIZ_MARGIN;

	  sys_pmu.b[c].text =
	       gkrellm_create_decal_text (pmu, "100", pmu->textstyle, style, x, -1, w);	/* -1 means use y default */
	  sys_pmu.b[c].text->y =
	       y + (style->margin + sys_pmu.b[c].icon->h - sys_pmu.b[0].text->h) / 2;
	  sys_pmu.b[c].icon->x = sys_pmu.b[c].modif->x = style->margin + HORIZ_MARGIN;
	  sys_pmu.b[c].icon->y = sys_pmu.b[c].modif->y = y;

	  y = sys_pmu.b[c].icon->y + sys_pmu.b[c].icon->h + VERT_MARGIN;
     }

     create_pmu_krells ();

     gkrellm_configure_panel (pmu, NULL, style);
     gkrellm_create_panel (vbox, pmu, gkrellm_bg_meter_image (0));

     if (sys_pmu.enabled) {
	  sys_pmu.visible = TRUE;
	  gkrellm_monitor_height_adjust (pmu->h);
     } else {
	  sys_pmu.visible = FALSE;
	  gtk_widget_hide (pmu->hbox);
     }
     if (first_create) {
	  gtk_signal_connect (GTK_OBJECT (pmu->drawing_area),
	       "expose_event", (GtkSignalFunc) pmu_expose_event, NULL);
	  gtk_signal_connect (GTK_OBJECT (pmu->drawing_area),
	       "button_release_event", (GtkSignalFunc) pmu_button_release, NULL);
     }

     update_battery_decals ();
     update_text_decals ();
     update_krells ();

     gkrellm_draw_layers (pmu);
}
