
/* GKrellM Powerbook PMU plugin 
|  Battery monitor using PMUD for Apple Powerbooks running Linux/PMac
|  Copyright (C) 2000  Joseph Palani Garcia
|  Author:   Joseph Palani Garcia jpgarcia@execpc.com
|  Version:  0.9
|  Latest versions might be found at:  http://www.execpc.com/~jpgarcia
|  or by following links from the GKrellM homepage at: http://gkrellm.net
|
|  This program is free software which I release under the GNU General Public
|  License. You may redistribute and/or modify this program under the terms
|  of that license as published by the Free Software Foundation; either
|  version 2 of the License, or (at your option) any later version.
|
|  This program is distributed in the hope that it will be useful,
|  but WITHOUT ANY WARRANTY; without even the implied warranty of
|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|  GNU General Public License for more details.
|
|  To get a copy of the GNU General Public License,  write to the
|  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "powerbook_pmu_priv.h"
#include "powerbook_pmu.h"

void
init_pmu_struct ()
{
     sys_pmu.b[1].available = sys_pmu.b[0].available = BATT_NA;
     sys_pmu.b[1].current = sys_pmu.b[0].current =
	  sys_pmu.b[1].percentage = sys_pmu.b[0].percentage =
	  sys_pmu.b[1].charging = sys_pmu.b[0].charging =
	  sys_pmu.time_left = sys_pmu.nbattery = sys_pmu.low_second_krell = sys_pmu.enabled =
	  sys_pmu.left_enabled = 0;
     sys_pmu.version = -1;
     sys_pmu.ac_is_on_line = sys_pmu.use_memory_style = 1;
}

int
init_pmu ()
{
     unsigned char vers;
     char c[100];

     pmud_socket (1);

     if (!sys_pmu.pmud.open)
	  return 0;

     read (sys_pmu.pmud.sock, c, 100);
     strtok (c, PMUD_INPUT_DELIM);
#ifdef VERBOSE_INSTALL
     printf ("Found PMUD version %s running on a ", strtok (0, PMUD_INPUT_DELIM));
#else
     strtok (0, PMUD_INPUT_DELIM);
#endif
     vers = atoi (strtok (0, PMUD_INPUT_DELIM));


     sys_pmu.version = vers;
     switch (vers) {
	  case PMU_VERSION_WALLSTREET:
	  case PMU_VERSION_LOMBARD:
	  sys_pmu.nbattery = 2;
	  sys_pmu.read_pmu = read_paddington_pmu;
#ifdef VERBOSE_INSTALL
	  printf ("%s\n", vers == PMU_VERSION_WALLSTREET ? "Wallstreet" : "Lombard");
	  printf ("Configured for Paddington with max 2 batteries\n");
#endif
	  break;
	  case PMU_VERSION_OHARE:
	  sys_pmu.nbattery = 1;
	  sys_pmu.read_pmu = read_ohare_pmu;
#ifdef VERBOSE_INSTALL
	  printf ("2400/3400/3500\n");
	  printf ("Configured for Ohare with max 1 battery\n");
	  printf ("WARNING: NOT IMPLEMENTED WELL\n");
#endif
	  break;
	  case PMU_VERSION_KEYLARGO:
	  sys_pmu.nbattery = 1;
	  sys_pmu.read_pmu = read_keylargo_pmu;
#ifdef VERBOSE_INSTALL
	  printf ("iBook\n");
	  printf ("Configured for Keylargo with max 1 battery\n");
	  printf ("WARNING: NOT IMPLEMENTED\n");
#endif
	  break;
	  default:
	  pmud_socket (0);
	  return 0;
     }

     (*(sys_pmu.read_pmu)) ();
     return (sys_pmu.nbattery > 0);
}

void
update_pmu ()
{
     int c;

     if (sys_pmu.visible == FALSE)
	  return;
     if (GK.two_second_tick) {
	  (*(sys_pmu.read_pmu)) ();

	  c = ((sys_pmu.left_enabled) && (sys_pmu.nbattery == 2)
	       && (sys_pmu.b[1].available == BATT_NO));
	  if (c != sys_pmu.left_mode_on) {
	       gint en = sys_pmu.enabled;

	       sys_pmu.left_mode_on = c;
	       sys_pmu.enabled = 0;
	       gkrellm_enable_visibility (sys_pmu.enabled, &sys_pmu.visible, pmu->hbox,
		    pmu->h);
	       create_pmu (sys_pmu.vbox, 0);
	       sys_pmu.enabled = en;
	       gkrellm_enable_visibility (sys_pmu.enabled, &sys_pmu.visible, pmu->hbox,
		    pmu->h);
	  }
	  if ((sys_pmu.nbattery == 1) || (sys_pmu.left_mode_on)) {
	       if (sys_pmu.b[0].available == BATT_NO)
		    sys_pmu.draw_current = 0;
	       /*          else if (sys_pmu.ac_is_on_line)
	          sys_pmu.draw_current = (get_current () == 0 ? 0 : 1);
	        */
	  }

	  update_battery_decals ();
	  update_text_decals ();
	  update_krells ();

	  gkrellm_draw_layers (pmu);
     }
}

Monitor monitor_pmu = {
     "PMU",
     0,
     create_pmu,
     update_pmu,
     create_pmu_tab,
     apply_pmu_config,

     save_pmu_config,
     load_pmu_config,
     PMU_CONFIG_KEYWORD,

     NULL,
     NULL,
     NULL,

     POWERBOOK_PMU_LOCATION,

     NULL,
     NULL
};

Monitor *
init_plugin (void)
{
     sys_pmu.enabled = FALSE;
     init_pmu_struct ();
     if (init_pmu ())
	  return &monitor_pmu;
     else
	  return NULL;
}
